@@ -1,4 +1,3 @@
-# $Id: CDDB.pm 6 2009-07-26 03:54:11Z rcaputo $
 # Documentation and Copyright exist after __END__
 
 package CDDB;
@@ -8,7 +7,7 @@ use strict;
 use vars qw($VERSION);
 use Carp;
 
-$VERSION = '1.20';
+$VERSION = '1.220';
 
 BEGIN {
 	if ($^O eq 'MSWin32') {
@@ -28,6 +27,8 @@ use Sys::Hostname;
 # TODO: Fetch the list from freedb.freedb.org, which is a round-robin
 # for all the others anyway.
 
+my $cddbp_host_selector = 0;
+
 my @cddbp_hosts = (
 	[ 'localhost'         => 8880 ],
 	[ 'freedb.freedb.org' => 8880 ],
@@ -343,31 +344,15 @@ sub connect {
 
 	HANDSHAKE: while ('true') {
 
-		# The host loop tries each possible host, in order.
+		# Loop through the CDDB protocol hosts list up to twice in order
+		# to find a server that will respond.  This implements a 2x retry.
 
-		HOST: while ('true') {
+		HOST: for (1..(@cddbp_hosts * 2)) {
 
 			# Hard disconnect here to prevent recursion.
 			delete $self->{handle};
 
-			# If no host has been selected, cycle to the next one in the
-			# list.  This destroys that list as it goes, but a successful
-			# connection later will restore the good host to the list.
-			# TODO: give bad hosts extra chances in case there are transient
-			# network problems.
-			if ($self->{host} eq '') {
-
-				# None of the servers worked.  Time to leave.
-				unless (@cddbp_hosts) {
-					$self->debug_print( 0, "--- all cddbp servers failed to answer" );
-					warn "No cddb protocol servers answer.  Is your network OK?\n"
-						unless $self->{debug};
-					return;
-				}
-
-				$cddbp_host = shift(@cddbp_hosts);
-				($self->{host}, $self->{port}) = @$cddbp_host;
-			}
+			($self->{host}, $self->{port}) = @{$cddbp_hosts[$cddbp_host_selector]};
 
 			# Assign the host we selected, and attempt a connection.
 			$self->debug_print(
@@ -388,8 +373,13 @@ sub connect {
 					0,
 					"--- error connecting to $self->{host} port $self->{port}: $!"
 				);
+
 				delete $self->{handle};
 				$self->{host} = $self->{port} = '';
+
+				# Try the next host in the list.  Wrap if necessary.
+				$cddbp_host_selector = 0 if ++$cddbp_host_selector > @cddbp_hosts;
+
 				next HOST;
 			}
 
@@ -400,12 +390,17 @@ sub connect {
 				0,
 				"+++ successfully connected to $self->{host} port $self->{port}"
 			);
-			push(@cddbp_hosts, $cddbp_host);
+
 			last HOST;
 		}
 
-		# This should not occur.
-		die unless defined $self->{handle};
+		# Tried the whole list twice without success?  Time to give up.
+		unless (defined $self->{handle}) {
+			$self->debug_print( 0, "--- all cddbp servers failed to answer" );
+			warn "No cddb protocol servers answer.  Is your network OK?\n"
+				unless $self->{debug};
+			return;
+		}
 
 		# Turn off buffering on the socket handle.
 		select((select($self->{handle}), $|=1)[0]);
@@ -1012,14 +1007,18 @@ sub submit_disc {
 	return;
 }
 
-###############################################################################
 1;
+
 __END__
 
 =head1 NAME
 
 CDDB.pm - a high-level interface to cddb protocol servers (freedb and CDDB)
 
+=head1 VERSION
+
+version 1.221
+
 =head1 SYNOPSIS
 
   use CDDB;
@@ -1100,6 +1099,12 @@ Disc details have been useful for generating CD catalogs, naming mp3
 files, printing CD liners, or even just playing discs in an automated
 jukebox.
 
+Despite the module's name, it connects to FreeDB servers by default.
+This began at version 1.04, when cddb.com changed its licensing model
+to support end-user applications, not third-party libraries.
+Connections to cddb.com may still work, and patches are welcome to
+maintain that functionality, but it's no longer officially supported.
+
 =head1 PUBLIC METHODS
 
 =over 4
@@ -1546,19 +1551,63 @@ Documented as being not documented.
 Please see the cddb.t program in the t (tests) directory.  It
 exercises every aspect of CDDB.pm, including submissions.
 
-=head1 BUGS
+=head1 COMPATIBILITY
 
-There are no known bugs, but see the README for things that need to be
-done.
+CDDB.pm uses standard Perl modules.  It has been tested at one point
+or another on OS/2, MacOS and FreeBSD systems, as well as the systems
+listed at:
+
+  http://testers.cpan.org/search?request=dist&dist=CDDB
+
+If you want to submit disc information to the CDDB, you will need to
+install two other modules:
+
+  Mail::Internet will allow CDDB.pm to send email submissions, and it
+  automagically includes Mail::Header.
+
+  MIME::QuotedPrint will allow CDDB.pm to send non-ASCII text
+  unscathed.  Currently only ISO-8859-1 and ASCII are supported.
+
+All other features will work without these modules.
+
+=head1 KNOWN TEST FAILURES
+
+The last test in the "make test" suite will try to send a sample
+submission to the CDDB if MailTools is present.  It expects to find an
+SMTP host in the SMTPHOST environment variable.  It will fall back to
+"mail" if SMTPHOST doesn't exist.  If neither works, the test will be
+skipped.  To see why it's skipped:
+
+  make test TEST_VERBOSE=1
+
+Some of the tests (most notably numbers 25, 27 and 29) compare data
+returned by a cddbp server against a stored copy of a previous query.
+These tests fail occasionally since the database is constantly in
+flux.  Starting with version 1.00, the test program uses fuzzy
+comparisons that should fail less.  Version 1.04 saw even fuzzier
+comparisons.  Please report any problems so they can be fixed.
+
+=head1 LINKS
+
+=head2 BUG TRACKER
+
+https://rt.cpan.org/Dist/Display.html?Status=Active&Queue=CDDB
+
+=head2 REPOSITORY
+
+http://github.com/rcaputo/cddb-perl
+http://gitorious.org/cddb-freedb-perl
+
+=head2 OTHER RESOURCES
+
+http://search.cpan.org/dist/CDDB/
 
 =head1 CONTACT AND COPYRIGHT
 
-Copyright 1998-2005 Rocco Caputo.  All rights reserved.  This program
+Copyright 1998-2013 Rocco Caputo.  All rights reserved.  This program
 is free software; you can redistribute it and/or modify it under the
 same terms as Perl itself.
 
-Rocco may be contacted at rcaputo@cpan.org.
-
 =cut
 
 # vim: sw=2 tw=70:
